@@ -17,8 +17,8 @@
 #' @param obj \emph{pspatfit} object fitted using \code{\link{pspatfit}} function. 
 #' @param listw should be a spatial neighbours list object created for example by \code{nb2listw} from \code{spdep} package. 
 #' It can also be a spatial weighting matrix of order (NxN) instead of a listw neighbours list object.
-#' @param conflevel numerical value for the confidence interval of the
-#'    impact functions. Default 0.95.
+#' @param alpha numerical value for the significance level of the pointwise 
+#'   confidence interval of the impact functions. Default 0.05.
 #' @param viewplot Default `TRUE` to plot impacts. If FALSE use \code{\link{plot_impactsnopar}} to plot impacts
 #' @param smooth Default `TRUE`. Whether to smooth fitted impacts or not.
 #' @param span span for the kernel of the smoothing (see \code{\link{loess}} 
@@ -147,21 +147,55 @@
 #'
 #' @examples
 #' ################################################
-#'  ###################### Examples using a panel data of rate of
-#'  ###################### unemployment for 103 Italian provinces in period 1996-2014.
+#' # Examples using spatial data of Ames Houses.
+#' ###############################################
+#' # Getting and preparing the data
+#' library(pspatreg)
+#' library(spdep)
+#' library(sf)
+#' ames <- AmesHousing::make_ames() # Raw Ames Housing Data
+#' ames_sf <- st_as_sf(ames, coords = c("Longitude", "Latitude"))
+#' ames_sf$Longitude <- ames$Longitude
+#' ames_sf$Latitude <- ames$Latitude
+#' ames_sf$lnSale_Price <- log(ames_sf$Sale_Price)
+#' ames_sf$lnLot_Area <- log(ames_sf$Lot_Area)
+#' ames_sf$lnTotal_Bsmt_SF <- log(ames_sf$Total_Bsmt_SF+1)
+#' ames_sf$lnGr_Liv_Area <- log(ames_sf$Gr_Liv_Area)
+#' ########### Constructing the spatial weights matrix
+#' ames_sf1 <- ames_sf[(duplicated(ames_sf$Longitude) == FALSE), ]
+#' coord_sf1 <- cbind(ames_sf1$Longitude, ames_sf1$Latitude)
+#' ID <- row.names(as(ames_sf1, "sf"))
+#' col_tri_nb <- tri2nb(coord_sf1)
+#' soi_nb <- graph2nb(soi.graph(col_tri_nb, 
+#'                             coord_sf1), 
+#'                    row.names = ID)
+#' lw_ames <- nb2listw(soi_nb, style = "W", 
+#'                     zero.policy = FALSE)
+#'                     
+#' form1 <- lnSale_Price ~ Fireplaces + Garage_Cars +
+#'           pspl(lnLot_Area, nknots = 20) + 
+#'           pspl(lnTotal_Bsmt_SF, nknots = 20) +
+#'           pspl(lnGr_Liv_Area, nknots = 20)    
+#' 
+#' gamsar <- pspatfit(form1, data = ames_sf1, 
+#'                    type = "sar", listw = lw_ames,
+#'                    method = "Chebyshev")
+#' summary(gamsar)
+#' \donttest{
+#' nparimpacts <- impactsnopar(gamsar, listw = lw_ames, viewplot = TRUE)
+#' ################################################
+#'  ######## Examples using a panel data of rate of
+#'  ######## unemployment for 103 Italian provinces in period 1996-2014.
 #' library(pspatreg)
 #' data(unemp_it, package = "pspatreg") 
 #' ## Wsp_it is a matrix. Create a neighboord list 
 #' lwsp_it <- spdep::mat2listw(Wsp_it)
-#' ## short sample for spatial pure case (2d)
-#' unemp_it_short <- unemp_it[unemp_it$year == 2019, ] 
 #' ######  No Spatial Trend: PSAR including a spatial 
 #' ######  lag of the dependent variable
-#' form1 <- unrate ~ partrate + agri + cons +
-#'                  pspl(serv, nknots = 15) +
-#'                  pspl(empgrowth, nknots = 20) 
+#' form1 <- unrate ~ partrate + agri + cons + empgrowth +
+#'                  pspl(serv, nknots = 15)
 #' gamsar <- pspatfit(form1, 
-#'                     data = unemp_it_short, 
+#'                     data = unemp_it, 
 #'                     type = "sar", 
 #'                     listw = lwsp_it)
 #'  summary(gamsar)
@@ -169,9 +203,10 @@
 #'  imp_nparvar <- impactsnopar(gamsar, 
 #'                              listw = lwsp_it, 
 #'                              viewplot = TRUE)
+#' }                              
 #'
 #' @export
-impactsnopar <- function(obj, listw = NULL, conflevel = 0.95, 
+impactsnopar <- function(obj, listw = NULL, alpha = 0.05, 
                          viewplot = TRUE, smooth = TRUE, 
                          span = c(0.1, 0.1, 0.2)) {
   type <- obj$type
@@ -196,13 +231,31 @@ impactsnopar <- function(obj, listw = NULL, conflevel = 0.95,
   # Build list of nonparametric variables
   variables <- varnopar
   for (i in 1:length(varnopar)) {
-    idx_varnopari <- str_detect(varnopar[i], colnames(obj$data))
-    variables[i] <- colnames(obj$data)[idx_varnopari]
+    varnopar_i <- stringr::str_extract(varnopar[i], colnames(obj$data))
+    varnopar_i <- varnopar_i[!is.na(varnopar_i)]
+    if (length(varnopar_i) > 1) {
+      # Variables with similar names in database 
+      # Compare from the longest to lowest names 
+      order_varnopar_i <- order(stringr::str_length(varnopar_i), 
+                                decreasing = TRUE)
+      for (j in 1:length(order_varnopar_i)) {
+        varnopar_ij <- varnopar_i[order_varnopar_i[j]]
+        idx_varnopar_ij <- str_detect(colnames(obj$data), varnopar_ij)
+        if (sum(idx_varnopar_ij) > 0) {
+          if (sum(idx_varnopar_ij) > 1) 
+            stop("Some variables share the same name in the database")
+          else {
+            variables[i] <- varnopar_ij
+            break
+          }
+        }
+      }
+    } else  variables[i] <- varnopar_i
   }
   fitsall <- fit_terms(obj, variables)
   fits <- fitsall$fitted_terms
   sefits <- fitsall$se_fitted_terms
-  crval <- qnorm((1 - conflevel)/2, mean = 0, 
+  crval <- qnorm(alpha/2, mean = 0, 
                  sd = 1, lower.tail = FALSE)
   fitsup <- fits + crval*sefits
   fitslow <- fits - crval*sefits
