@@ -43,26 +43,86 @@
 #'
 #' @examples
 #' library(pspatreg)
-#' data(unemp_it)
-#' ## short sample for spatial pure case (2d)
-#' unemp_it_short <- unemp_it[unemp_it$year == 2019, ]
-#'  ######################  GAM pure
-#' form1 <- unrate ~ partrate + agri + cons +
-#'                  pspl(serv, nknots = 15) +
-#'                  pspl(empgrowth, nknots = 20)
-#' gampure <- pspatfit(form1, data = unemp_it_short)
-#' summary(gampure)
-#' #########  GAM pure with spatial non-parametric term
-#'  \donttest{
-#'  geosp1 <- pspatfit(form1, data = unemp_it_short)
-#'  summary(geosp1)
+#' ###############################################
+#' # Examples using spatial data of Ames Houses.
+#' ###############################################
+#' library(spdep)
+#' library(sf)
+#' ames <- AmesHousing::make_ames() # Raw Ames Housing Data
+#' ames_sf <- st_as_sf(ames, coords = c("Longitude", "Latitude"))
+#' ames_sf$Longitude <- ames$Longitude
+#' ames_sf$Latitude <- ames$Latitude
+#' ames_sf$lnSale_Price <- log(ames_sf$Sale_Price)
+#' ames_sf$lnLot_Area <- log(ames_sf$Lot_Area)
+#' ames_sf$lnTotal_Bsmt_SF <- log(ames_sf$Total_Bsmt_SF+1)
+#' ames_sf$lnGr_Liv_Area <- log(ames_sf$Gr_Liv_Area)
+#' ########### Constructing the spatial weights matrix
+#' ames_sf1 <- ames_sf[(duplicated(ames_sf$Longitude) == FALSE), ]
+#' coord_sf1 <- cbind(ames_sf1$Longitude, ames_sf1$Latitude)
+#' ID <- row.names(as(ames_sf1, "sf"))
+#' col_tri_nb <- tri2nb(coord_sf1)
+#' soi_nb <- graph2nb(soi.graph(col_tri_nb, 
+#'                             coord_sf1), 
+#'                    row.names = ID)
+#' lw_ames <- nb2listw(soi_nb, style = "W", 
+#'                     zero.policy = FALSE)
 #' 
+#' ####  GAM pure with pspatreg
+#' form1 <- lnSale_Price ~ Fireplaces + Garage_Cars +
+#'           pspl(lnLot_Area, nknots = 20) + 
+#'           pspl(lnTotal_Bsmt_SF, nknots = 20) +
+#'           pspl(lnGr_Liv_Area, nknots = 20)    
+#' gampure <- pspatfit(form1, data = ames_sf1)
+#' summary(gampure)
+#' #########  GAM pure with spatial trend (2d)
+#'  \donttest{
+#' #####################  GAM + SAR Model
+#' gamsar <- pspatfit(form1, data = ames_sf1, 
+#'                    type = "sar", listw = lw_ames,
+#'                    method = "Chebyshev")
+#' summary(gamsar)
+#' ### Models with 2d spatial trend
+#' form2 <- lnSale_Price ~ Fireplaces + Garage_Cars +
+#'           pspl(lnLot_Area, nknots = 20) + 
+#'           pspl(lnTotal_Bsmt_SF, nknots = 20) +
+#'           pspl(lnGr_Liv_Area, nknots = 20) +
+#'           pspt(Longitude, Latitude, 
+#'                nknots = c(10, 10), 
+#'                psanova = FALSE)
+#' #####################  GAM + GEO Model
+#' gamgeo2d <- pspatfit(form2, data = ames_sf1)
+#' summary(gamgeo2d)
+#' 
+#' gamgeo2dsar <- pspatfit(form2, data = ames_sf1,
+#'                         type = "sar", 
+#'                         listw = lw_ames, 
+#'                         method = "Chebyshev")
+#' summary(gamgeo2dsar)
+#' ### Models with psanova 2d spatial trend
+#' form3 <- lnSale_Price ~ Fireplaces + Garage_Cars +
+#'           pspl(lnLot_Area, nknots = 20) + 
+#'           pspl(lnTotal_Bsmt_SF, nknots = 20) +
+#'           pspl(lnGr_Liv_Area, nknots = 20) +
+#'           pspt(Longitude, Latitude, 
+#'                nknots = c(10, 10), 
+#'                psanova = TRUE)
+#' gamgeo2danovasar <- pspatfit(form3, data = ames_sf1,
+#'                         type = "sar", 
+#'                         listw = lw_ames, method = "Chebyshev")
+#' summary(gamgeo2danovasar)
 #'  ###############################################
+#' ###################### Examples using a panel data of rate of
+#' ###################### unemployment for 103 Italian provinces in 1996-2019.
+#'  ###############################################
+#' ## load spatial panel and Wsp_it
+#' ## 103 Italian provinces. Period 1996-2019
+#' data(unemp_it, package = "pspatreg")
+#' ## Wsp_it is a matrix. Create a neighboord list 
+#' lwsp_it <- spdep::mat2listw(Wsp_it)
 #'  ### Spatio-temporal semiparametric ANOVA model 
 #'  ### Interaction terms f12,f1t,f2t and f12t with nested basis
 #'  ### Remark: nest_sp1, nest_sp2 and nest_time must be divisors of nknots
-#'  
-#'  form2 <- unrate ~ partrate + agri + cons +
+#'  form4 <- unrate ~ partrate + agri + cons +
 #'                    pspl(serv, nknots = 15) + 
 #'                    pspl(empgrowth, nknots = 20) +
 #'                    pspt(long, lat, year, 
@@ -71,13 +131,13 @@
 #'                         nest_sp1 = c(1, 2, 2), 
 #'                         nest_sp2 = c(1, 2, 2),
 #'                         nest_time = c(1, 2, 2))
-#'  sptanova <- pspatfit(form2, data = unemp_it)
+#'  sptanova <- pspatfit(form4, data = unemp_it)
 #'  summary(sptanova)
 #'  
 #' 
 #'  ################################################  
 #'  ### Interaction terms f1t not included in ANOVA decomposition
-#'  form3 <- unrate ~ partrate + agri + cons +
+#'  form5 <- unrate ~ partrate + agri + cons +
 #'                    pspl(serv, nknots = 15) + 
 #'                    pspl(empgrowth, nknots=20) +
 #'                    pspt(long, lat, year, 
@@ -87,8 +147,12 @@
 #'                         nest_sp2 = c(1, 2, 3),
 #'                         nest_time = c(1, 2, 2), 
 #'                         f1t_int = FALSE)
-#'  sptanova2 <- pspatfit(form3, data = unemp_it)
-#'  summary(sptanova2)
+#'  ## Add sar specification and ar1 temporal correlation 
+#'  sptanova2_sar_ar1 <- pspatfit(form5, data = unemp_it, 
+#'                               listw = lwsp_it, 
+#'                               type = "sar",
+#'                               cor = "ar1")
+#' summary(sptanova2_sar_ar1)                                 
 #'  }
 NULL
 
@@ -97,9 +161,7 @@ NULL
 #'   
 #' @param x Name of the covariate.
 #' @param xl Minimum of the interval for the continuous covariate.
-#'   Default = min(x) - 0.01.
 #' @param xr Maximum of the interval for the continuous covariate.
-#'   Default = max(x) - 0.01.
 #' @param nknots Number of knots for spline basis. Default = 10.
 #' @param bdeg Order of the B-spline basis. Default = 3.
 #' @param pord Order of the penalty for the difference matrix 
@@ -109,7 +171,7 @@ NULL
 #'   fixed part is given by \eqn{X = B*U_n} where \emph{B} is the
 #'   B-spline basis matrix and \emph{U_n} is the nullspace basis of the 
 #'   penalty matrix. If \code{decom = 2} the fixed part is given by
-#'   \eqn{X = [1|x|...|x^(pord-1)] }. Default = 2.
+#'   \eqn{X = [1|x|...|x^(pord-1)] }. Default = 1.
 #'   
 #' @description 
 #'   \code{pspl()}: This function allows the inclusion of terms for
@@ -126,15 +188,17 @@ NULL
 #'  }
 #' 
 #' @export
-pspl <- function(x, xl = min(x) - 0.01, xr = max(x) + 0.01,
-                nknots = 10, bdeg = 3, pord = 2, decom = 2){
+pspl <- function(x, xl = min(x) - 0.01*abs(min(x)), 
+                 xr = max(x) + 0.01*abs(max(x)),
+                nknots = 10, bdeg = 3, pord = 2, 
+                decom = 3){
   dx <- (xr - xl)/nknots
   knots <- seq(xl - bdeg*dx, xr + bdeg*dx, by = dx)
   B <- spline.des(knots, x, bdeg + 1, 0*x)$design
   a <- list(nknots = nknots, knots = knots, bdeg = bdeg, 
-            pord = pord, decom = decom)
+            pord = pord, decom = decom, x = x)
   attributes(B) <- c(attributes(B), a)
-  class(B) <- c("bs","basis","matrix")
+  class(B) <- c("bs", "basis", "matrix")
   B
 }
 
@@ -159,17 +223,11 @@ pspl <- function(x, xl = min(x) - 0.01, xr = max(x) + 0.01,
 #'   coordinates before the estimation of semiparametric model. 
 #'   Default = `TRUE`   
 #' @param xl_sp1 Minimum of the interval for the first spatial coordinate.
-#'   Default = min(sp1) - 0.01.    
 #' @param xr_sp1 Maximum of the interval for the first spatial coordinate.
-#'   Default = max(sp1) + 0.01.     
 #' @param xl_sp2 Minimum of the interval for the second spatial coordinate.
-#'   Default = min(sp2) - 0.01.    
 #' @param xr_sp2 Maximum of the interval for the second spatial coordinate.
-#'   Default = max(sp2) + 0.01.  
 #' @param xl_time Minimum of the interval for the temporal coordinate.
-#'   Default = min(time) - 0.01.    
 #' @param xr_time Maximum of the interval for the temporal coordinate.
-#'   Default = max(time) + 0.01.  
 #' @param nknots Vector including the number of knots of each 
 #'   coordinate for spline bases. Default = c(10,10,5). The order of the knots
 #'   in the vector follows the order of the specified spatio-temporal parameters 
@@ -236,11 +294,14 @@ pspl <- function(x, xl = min(x) - 0.01, xr = max(x) + 0.01,
 #'    
 #' @export
 pspt <- function(sp1, sp2, time = NULL, scale = TRUE, ntime = NULL,
-                xl_sp1 = min(sp1) - 0.01, xr_sp1 = max(sp1) + 0.01,
-                xl_sp2 = min(sp2) - 0.01, xr_sp2 = max(sp2) + 0.01,
-                xl_time = min(time)- 0.01, xr_time = max(time) + 0.01,
+                xl_sp1 = min(sp1) - 0.01*abs(min(sp1)), 
+                xr_sp1 = max(sp1) + 0.01*abs(max(sp1)),
+                xl_sp2 = min(sp2) - 0.01*abs(min(sp2)), 
+                xr_sp2 = max(sp2) + 0.01*abs(max(sp2)),
+                xl_time = min(time) - 0.01*abs(min(time)), 
+                xr_time = max(time) + 0.01*abs(max(time)),
                 nknots = c(10,10,5), bdeg = c(3,3,3), pord = c(2,2,2),
-                decom = 1, psanova = FALSE,
+                decom = 3, psanova = FALSE,
                 nest_sp1 = 1, nest_sp2 = 1, nest_time = 1,
                 f1_main = TRUE, f2_main = TRUE, ft_main = TRUE,
                 f12_int = TRUE, f1t_int = TRUE, f2t_int = TRUE,
@@ -249,54 +310,56 @@ pspt <- function(sp1, sp2, time = NULL, scale = TRUE, ntime = NULL,
   nsp <- length(sp1)
   if (!is.null(time)) {
     ntime <- length(unique(time))
-    # if (length(sp1) != length(time)) stop("variables must have same length")
-    # if (is.null(ntime)) stop("ntime is needed as argument")
   }
   if (scale){
-    sp1 <- scale(sp1); sp2 <- scale(sp2)
-    xl_sp1=min(sp1)-0.01; xr_sp1=max(sp1)+0.01
-    xl_sp2=min(sp2)-0.01; xr_sp2=max(sp2)+0.01
+    sp1 <- scale(sp1)
+    sp2 <- scale(sp2)
+    xl_sp1 = min(sp1) - 0.01
+    xr_sp1 = max(sp1) + 0.01
+    xl_sp2 = min(sp2) - 0.01
+    xr_sp2 = max(sp2) + 0.01
     if (!is.null(time)) {
       time <- scale(time)
-      xl_time=min(time)-0.01; xr_time=max(time)+0.01
+      xl_time = min(time) - 0.01
+      xr_time = max(time) + 0.01
     }
   }
   B <- NULL
   if (psanova){
     decom <- 2
-    if (length(nest_sp1) != length(nest_sp2)){
+    if (length(nest_sp1) != length(nest_sp2)) 
       stop("nest_sp1 and nest_sp2 must have same length")
+    if (!is.null(time)) {
+      if (length(nest_sp1) != length(nest_time)) 
+        stop("nest_sp and nest_time must have same length")
     }
-    if (!is.null(time)){
-      if (length(nest_sp1) != length(nest_time)) {
-         stop("nest_sp and nest_time must have same length")
-       }
-    }
-    if (length(nest_sp1)==1) nest_sp1 <- rep(nest_sp1,3)
-    if (length(nest_sp2)==1) nest_sp2 <- rep(nest_sp2,3)
-    if (!is.null(time)){ # Efectos Interacción hasta orden 3
-      if (length(nest_time)==1) nest_time <- rep(nest_time,3)
+    if (length(nest_sp1) == 1) nest_sp1 <- rep(nest_sp1,3)
+    if (length(nest_sp2) == 1) nest_sp2 <- rep(nest_sp2,3)
+    if (!is.null(time)) { # Efectos Interacción hasta orden 3
+      if (length(nest_time)==1) nest_time <- rep(nest_time, 3)
       order_effects <- 1:3
     } else { # Efectos Interacción hasta orden 2
     order_effects <- 1:2
     nest_time <- rep(1,length(nest_sp1))
     }
     for (i in 1:length(order_effects)) {
-       Bsp1 <- pspl(sp1,xl=xl_sp1,xr=xr_sp1,
-                   nknots=nknots[1]/nest_sp1[i],
-                   bdeg=bdeg[1],pord=pord[1],decom=decom)
-       colnames(Bsp1) <- paste("Bsp1",1:ncol(Bsp1),sep=".")
-       Bsp2 <- pspl(sp2,xl=xl_sp2,xr=xr_sp2,
-                    nknots=nknots[2]/nest_sp2[i],
-                    bdeg=bdeg[2],pord=pord[2],decom=decom)
-       colnames(Bsp2) <- paste("Bsp2",1:ncol(Bsp2),sep=".")
+       Bsp1 <- pspl(sp1, xl = xl_sp1, xr = xr_sp1,
+                   nknots = nknots[1]/nest_sp1[i],
+                   bdeg = bdeg[1], pord = pord[1], 
+                   decom = decom)
+       colnames(Bsp1) <- paste("Bsp1", 1:ncol(Bsp1), sep=".")
+       Bsp2 <- pspl(sp2, xl = xl_sp2, xr = xr_sp2,
+                    nknots = nknots[2]/nest_sp2[i],
+                    bdeg = bdeg[2], pord = pord[2], decom=decom)
+       colnames(Bsp2) <- paste("Bsp2",1:ncol(Bsp2), sep=".")
        Bi <- cbind(Bsp1,Bsp2)
        if(!is.null(time)) {
-         Btime <- pspl(time,xl=xl_time,xr=xr_time,
-                      nknots=nknots[3]/nest_time[i],
-                      bdeg=bdeg[3],pord=pord[3],decom=decom)
-         colnames(Btime) <- paste("Btime",1:ncol(Btime),sep=".")
-         Bi <- cbind(Bi,Btime)
+         Btime <- pspl(time, xl = xl_time, xr = xr_time,
+                      nknots = nknots[3]/nest_time[i],
+                      bdeg = bdeg[3], pord = pord[3],
+                      decom = decom)
+         colnames(Btime) <- paste("Btime", 1:ncol(Btime), sep = ".")
+         Bi <- cbind(Bi, Btime)
        } else {
          ft_main <- f1t_int <- f2t_int <- f12t_int <- FALSE
          nest_time <- NULL; bdeg <- bdeg[1:2]; pord <- pord[1:2]
@@ -304,65 +367,63 @@ pspt <- function(sp1, sp2, time = NULL, scale = TRUE, ntime = NULL,
        if(i==1) colnames(Bi) <- paste(colnames(Bi),"main",sep=".")
        if(i==2) colnames(Bi) <- paste(colnames(Bi),"int2ord",sep=".")
        if(i==3) colnames(Bi) <- paste(colnames(Bi),"int3ord",sep=".")
-       B <- cbind(B,Bi)
+       B <- cbind(B, Bi)
     }
   } else { # PSANOVA=FALSE
     f1_main <- f2_main <- ft_main <- f12_int <- f1t_int <- FALSE
     f2t_int <- f12t_int <- FALSE
-    Bsp1 <- pspl(sp1,xl=xl_sp1,xr=xr_sp1,nknots=nknots[1],
-                 bdeg=bdeg[1],pord=pord[1],decom=decom)
-    colnames(Bsp1) <- paste("Bsp1",1:ncol(Bsp1),sep=".")
-    Bsp2 <- pspl(sp2,xl=xl_sp2,xr=xr_sp2, nknots=nknots[2],
-                 bdeg=bdeg[2],pord=pord[2],decom=decom)
-    colnames(Bsp2) <- paste("Bsp2",1:ncol(Bsp2),sep=".")
-    B <- cbind(Bsp1,Bsp2)
+    Bsp1 <- pspl(sp1, xl = xl_sp1, xr = xr_sp1, nknots = nknots[1],
+                 bdeg = bdeg[1], pord = pord[1], decom = decom)
+    colnames(Bsp1) <- paste("Bsp1", 1:ncol(Bsp1), sep = ".")
+    Bsp2 <- pspl(sp2, xl = xl_sp2, xr = xr_sp2, nknots = nknots[2],
+                 bdeg = bdeg[2], pord = pord[2], decom = decom)
+    colnames(Bsp2) <- paste("Bsp2", 1:ncol(Bsp2), sep = ".")
+    B <- cbind(Bsp1, Bsp2)
     if (!is.null(time)){
-       Btime <- pspl(time,xl=xl_time,xr=xr_time, nknots=nknots[3],
-                 bdeg=bdeg[3],pord=pord[3],decom=decom)
-       colnames(Btime) <- paste("Btime",1:ncol(Btime),sep=".")
-       B <- cbind(B,Btime)
+       Btime <- pspl(time, xl = xl_time, xr = xr_time, nknots = nknots[3],
+                 bdeg = bdeg[3], pord = pord[3], decom = decom)
+       colnames(Btime) <- paste("Btime", 1:ncol(Btime), sep = ".")
+       B <- cbind(B, Btime)
     } else { nest_time <- NULL; bdeg <- bdeg[1:2]; pord <- pord[1:2] }
   }
   a <- list(sp1 = sp1, sp2 = sp2, time = time, ntime = ntime,
             nknots = nknots, bdeg = bdeg, pord = pord, decom = decom,
-            psanova = psanova, nest_sp1 = nest_sp1, 
+            psanova = psanova, nest_sp1 = nest_sp1,
             nest_sp2 = nest_sp2, nest_time = nest_time,
             f1_main = f1_main, f2_main = f2_main, ft_main = ft_main,
             f12_int = f12_int, f1t_int = f1t_int, f2t_int = f2t_int,
             f12t_int = f12t_int)
-  attributes(B) <- c(attributes(B),a)
-  class(B) <- c("bs","basis","matrix")
+  attributes(B) <- c(attributes(B), a)
+  class(B) <- c("bs", "basis", "matrix")
   B
 }
 
-#####################################################################################
-
-B_XZ <- function (B, x=NULL, pord=2, decom=1){
+################################################################
+B_XZ <- function(B, x = NULL, pord = 2 , decom = 3) {
   m <- ncol(B)
   n <- nrow(B)
-  D <- diff(diag(m), differences=pord)
+  D <- diff(diag(m), differences = pord)
   P.svd <- svd(crossprod(D))
-  U <- (P.svd$u)[,1:(m-pord)] # eigenvectors
+  U <- (P.svd$u)[, 1:(m-pord)] # eigenvectors
   d <- (P.svd$d)[1:(m-pord)]  # eigenvalues
   Z <- B %*% U
   if(decom == 1) {
-    X <- B %*% ((P.svd$u)[,-(1:(m-pord))])
+    X <- B %*% ((P.svd$u)[, -(1:(m-pord))])
   } else if (decom == 2){
+    if (is.null(x)) stop("decom = 2 and is.null(x) = TRUE")
     X <- NULL
-    for(i in 0:(pord-1)){ X <- cbind(X,x^i) }
+    for(i in 1:(pord)){ X <- cbind(X, x^(i-1)) }
+  }
+  else if (decom == 3) {
+    X <- B %*% ((P.svd$u)[, -(1:(m - pord))])
+    D.temp <- sweep(X, 2, colMeans(X))
+    Xf <- svd(crossprod(D.temp))$u[, ncol(D.temp):1]
+    X <- X %*% Xf
   }
-  # else if(decom == 3) {
-  #   Xf <- NULL
-  #   for(i in 0:(pord-1)){
-  #     Xf <- cbind(Xf,knots[-c((1:pord),
-  #         (length(knots)- pord + 1):length(knots))]^i)
-  #   }
-  #   X <- B %*% Xf
-  # }
   list(X = X, Z = Z, d = d, B = B, m = m, D = D, U = P.svd$u)
 }
 
-#####################################################################################
+#############################################################
 
 Bspt <- function(sp1, sp2, time, nfull, ntime, psanova,
                  Bi, bdegspt){
@@ -392,7 +453,7 @@ Bspt <- function(sp1, sp2, time, nfull, ntime, psanova,
     Bsptlist <- list(Bsp1 = Bsp1, Bsp2 = Bsp2, Btime = Btime)
     #spt_names <- c("sp1","sp2","time")
     rm(Bsp1,Bsp2,Btime)
-  } else {# PSANOVA=TRUE
+  } else {# PSANOVA = TRUE
     Bsp1_col <- colnames(Bsp1)
     Bsp1_col_main <- Bsp1_col[grepl("main", Bsp1_col)]
     Bsp1_col_int2ord <- Bsp1_col[grepl("int2ord", Bsp1_col)]
@@ -437,24 +498,23 @@ Bspt <- function(sp1, sp2, time, nfull, ntime, psanova,
               Bsptlist = Bsptlist)
 }
 
-#####################################################################################
-
-B_XZ_spt <- function(sp1,sp2,time,pordspt,psanova,decomspt,
-                     f1_main,f2_main,ft_main,
-                     f12_int,f1t_int,f2t_int,f12t_int,Bsptlist){
+################################################################
 
+B_XZ_spt <- function(sp1, sp2, time, pordspt, psanova, decomspt,
+                     f1_main, f2_main, ft_main,
+                     f12_int, f1t_int, f2t_int, f12t_int, 
+                     Bsptlist) {
   Xsptlist <- Zsptlist <- dsptlist <- csptlist <- list()
   names_Xj <- names_Zj <- names_dj <- names_cj <-NULL
   cont <- 1
-  for (j in 1:length(names(Bsptlist)))
-  {
+  for (j in 1:length(names(Bsptlist))) {
     Bj <- Bsptlist[[j]]
     if (is.null(Bj))  next
     name_Bj <- names(Bsptlist)[j]
-    spt_term <- sub("B","",name_Bj)
-    is_sp1_term <- grepl("sp1",spt_term)
-    is_sp2_term <- grepl("sp2",spt_term)
-    is_time_term <- grepl("time",spt_term)
+    spt_term <- sub("B","", name_Bj)
+    is_sp1_term <- grepl("sp1", spt_term)
+    is_sp2_term <- grepl("sp2", spt_term)
+    is_time_term <- grepl("time", spt_term)
     if (is_sp1_term){
       x_term <- sp1
       pord_term <- pordspt[1]
@@ -467,39 +527,35 @@ B_XZ_spt <- function(sp1,sp2,time,pordspt,psanova,decomspt,
     }
     #if (!is.null(Bj)){
       #print(name_Bj)
-      BtoXZ <- B_XZ(Bj,x=x_term,pord=pord_term,decom=decomspt)
+      BtoXZ <- B_XZ(Bj, x = x_term,
+                    pord = pord_term,
+                    decom = decomspt)
       Xj <- BtoXZ$X
-      names_Xj <- c(names_Xj,paste("X",spt_term,sep=""))
+      names_Xj <- c(names_Xj, paste("X", spt_term, sep = ""))
       Xsptlist[[cont]] <- Xj
       Zj <- BtoXZ$Z
-      names_Zj <- c(names_Zj,paste("Z",spt_term,sep=""))
+      names_Zj <- c(names_Zj, paste("Z", spt_term, sep = ""))
       Zsptlist[[cont]] <- Zj
       dj <- BtoXZ$d
-      names_dj <- c(names_dj,paste(spt_term,sep=""))
+      names_dj <- c(names_dj, paste(spt_term, sep = ""))
       dsptlist[[cont]] <- dj
-      names_cj <- c(names_cj,paste(spt_term,sep=""))
+      names_cj <- c(names_cj, paste(spt_term, sep = ""))
       csptlist[[cont]] <- ncol(Bj)
       cont <- cont + 1
     #} else cont <- NULL
   } # end for (j in 1:length(names(Bsptlist)))
-  rm(Bj,name_Bj,x_term,pord_term,spt_term,cont)
-  rm(is_sp1_term,is_sp2_term,is_time_term)
-
+  rm(Bj, name_Bj, x_term, pord_term, spt_term, cont)
+  rm(is_sp1_term, is_sp2_term, is_time_term)
   names(Xsptlist) <- names_Xj
   names(Zsptlist) <- names_Zj
   names(dsptlist) <- names_dj
   names(csptlist) <- names_cj
-  rm(Xj,Zj,dj,names_Xj,names_Zj,names_dj,names_cj)
-
-  # REVISAR ESTA PARTE....
-  # g1u <- rep(1,pord[2])%x%d1
-  # g2u <- d2%x%rep(1,pord[1])
-  # g1b <- rep(1,c2-pord[2])%x%d1
-  # g2b <- d2%x%rep(1,c1-pord[1])
-
+  rm(Xj, Zj, dj, names_Xj, names_Zj, names_dj, names_cj)
   if (!psanova){  # PS-ANOVA=FALSE
-    X1 <- Xsptlist$Xsp1; X2 <- Xsptlist$Xsp2
-    Z1 <- Zsptlist$Zsp1; Z2 <- Zsptlist$Zsp2
+    X1 <- Xsptlist$Xsp1
+    X2 <- Xsptlist$Xsp2
+    Z1 <- Zsptlist$Zsp1
+    Z2 <- Zsptlist$Zsp2
     if (!is.null(time)) { #SPATIO-TEMPORAL TREND. NO PS-ANOVA
       Xt <- Xsptlist$Xtime
       Zt <- Zsptlist$Ztime
@@ -522,66 +578,70 @@ B_XZ_spt <- function(sp1,sp2,time,pordspt,psanova,decomspt,
     }
     rm(X1,Z1)
   } else { # PS-ANOVA=TRUE
-    X1 <- Xsptlist$Xsp1_main; X2 <- Xsptlist$Xsp2_main
-    Z1 <- Zsptlist$Zsp1_main; Z2 <- Zsptlist$Zsp2_main
-    Z1.2 <- Zsptlist$Zsp1_int2ord; Z2.2 <- Zsptlist$Zsp2_int2ord
-    one1 <- X1[,1,drop=FALSE]
-    one2 <- X2[,1,drop=FALSE]
-    x1 <- X1[,-1,drop=FALSE]
-    x2 <- X2[,-1,drop=FALSE]
+    X1 <- Xsptlist$Xsp1_main
+    X2 <- Xsptlist$Xsp2_main
+    Z1 <- Zsptlist$Zsp1_main
+    Z2 <- Zsptlist$Zsp2_main
+    Z1.2 <- Zsptlist$Zsp1_int2ord
+    Z2.2 <- Zsptlist$Zsp2_int2ord
+    one1 <- X1[, 1, drop = FALSE]
+    one2 <- X2[, 1, drop = FALSE]
+    x1 <- X1[, -1, drop = FALSE]
+    x2 <- X2[, -1, drop = FALSE]
     if (!is.null(time)) { #SPATIO-TEMPORAL TREND. PS-ANOVA
-      Xt <- Xsptlist$Xtime_main;
-      Zt <- Zsptlist$Ztime_main; Zt.2 <- Zsptlist$Ztime_int2ord
-      Z1.3 <- Zsptlist$Zsp1_int3ord; Z2.3 <- Zsptlist$Zsp2_int3ord
+      Xt <- Xsptlist$Xtime_main
+      Zt <- Zsptlist$Ztime_main
+      Zt.2 <- Zsptlist$Ztime_int2ord
+      Z1.3 <- Zsptlist$Zsp1_int3ord
+      Z2.3 <- Zsptlist$Zsp2_int3ord
       Zt.3 <- Zsptlist$Ztime_int3ord
-      onet <- Xt[,1,drop=FALSE]
-      xt <- Xt[,-1,drop=FALSE]
-      Xones <- kronecker(Rten2(one1,one2),onet)
-      X_f1_main <-  kronecker(Rten2(x1,one2),onet)
-      X_f2_main <-  kronecker(Rten2(one1,x2),onet)
-      X_ft_main <-  kronecker(Rten2(one1,one2),xt)
+      onet <- Xt[, 1, drop = FALSE]
+      xt <- Xt[, -1, drop = FALSE]
+      Xones <- kronecker(Rten2(one1,one2), onet)
+      X_f1_main <- kronecker(Rten2(x1,one2), onet)
+      X_f2_main <- kronecker(Rten2(one1,x2),onet)
+      X_ft_main <- kronecker(Rten2(one1,one2),xt)
       X_f12_int <- kronecker(Rten2(x1,x2),onet)
       X_f1t_int <- kronecker(Rten2(x1,one2),xt)
       X_f2t_int <- kronecker(Rten2(one1,x2),xt)
       X_f12t_int <- kronecker(Rten2(x1,x2),xt)
-      colnames(Xones) <- c("Intercept")
-      colnames(X_f1_main) <- paste("X_f1_main",1:ncol(X_f1_main),sep=".")
-      colnames(X_f2_main) <- paste("X_f2_main",1:ncol(X_f2_main),sep=".")
-      colnames(X_ft_main) <- paste("X_ft_main",1:ncol(X_ft_main),sep=".")
-      colnames(X_f12_int) <- paste("X_f12_int",1:ncol(X_f12_int),sep=".")
-      colnames(X_f1t_int) <- paste("X_f1t_int",1:ncol(X_f1t_int),sep=".")
-      colnames(X_f2t_int) <- paste("X_f2t_int",1:ncol(X_f2t_int),sep=".")
-      colnames(X_f12t_int) <- paste("X_f12t_int",1:ncol(X_f12t_int),sep=".")
+      colnames(Xones) <- c("(Intercept)")
+      colnames(X_f1_main) <- paste("X_f1_main", 1:ncol(X_f1_main), sep = ".")
+      colnames(X_f2_main) <- paste("X_f2_main", 1:ncol(X_f2_main), sep = ".")
+      colnames(X_ft_main) <- paste("X_ft_main", 1:ncol(X_ft_main), sep = ".")
+      colnames(X_f12_int) <- paste("X_f12_int", 1:ncol(X_f12_int), sep = ".")
+      colnames(X_f1t_int) <- paste("X_f1t_int", 1:ncol(X_f1t_int), sep = ".")
+      colnames(X_f2t_int) <- paste("X_f2t_int", 1:ncol(X_f2t_int), sep = ".")
+      colnames(X_f12t_int) <- paste("X_f12t_int", 1:ncol(X_f12t_int), sep = ".")
       Xspt <- Xones
-      if(f1_main) Xspt <- cbind(Xspt,X_f1_main)
-      if(f2_main) Xspt <- cbind(Xspt,X_f2_main)
-      if(ft_main) Xspt <- cbind(Xspt,X_ft_main)
-      if(f12_int) Xspt <- cbind(Xspt,X_f12_int)
-      if(f1t_int) Xspt <- cbind(Xspt,X_f1t_int)
-      if(f2t_int) Xspt <- cbind(Xspt,X_f2t_int)
-      if(f12t_int) Xspt <- cbind(Xspt,X_f12t_int)
+      if(f1_main) Xspt <- cbind(Xspt, X_f1_main)
+      if(f2_main) Xspt <- cbind(Xspt, X_f2_main)
+      if(ft_main) Xspt <- cbind(Xspt, X_ft_main)
+      if(f12_int) Xspt <- cbind(Xspt, X_f12_int)
+      if(f1t_int) Xspt <- cbind(Xspt, X_f1t_int)
+      if(f2t_int) Xspt <- cbind(Xspt, X_f2t_int)
+      if(f12t_int) Xspt <- cbind(Xspt, X_f12t_int)
       Z_f1_main <- kronecker(Rten2(Z1, one2), onet) # g1u
       Z_f2_main <- kronecker(Rten2(one1, Z2), onet) # g2u
       Z_ft_main <- kronecker(Rten2(one1, one2), Zt) # g3u
-      colnames(Z_f1_main) <- paste("Z_f1_main",1:ncol(Z_f1_main),sep=".")
-      colnames(Z_f2_main) <- paste("Z_f2_main",1:ncol(Z_f2_main),sep=".")
-      colnames(Z_ft_main) <- paste("Z_ft_main",1:ncol(Z_ft_main),sep=".")
-
+      colnames(Z_f1_main) <- paste("Z_f1_main", 1:ncol(Z_f1_main), sep = ".")
+      colnames(Z_f2_main) <- paste("Z_f2_main", 1:ncol(Z_f2_main), sep = ".")
+      colnames(Z_ft_main) <- paste("Z_ft_main", 1:ncol(Z_ft_main), sep = ".")
       # g12u | g21u | g12b+g21b
       Z_f12_int <- cbind(kronecker(Rten2(Z1.2, x2), onet),
                      kronecker(Rten2(x1, Z2.2), onet),
                      kronecker(Rten2(Z1.2, Z2.2), onet))
-      colnames(Z_f12_int) <- paste("Z_f12_int",1:ncol(Z_f12_int),sep=".")
+      colnames(Z_f12_int) <- paste("Z_f12_int", 1:ncol(Z_f12_int), sep = ".")
       # g13u | g31u | g13b+g31b
       Z_f1t_int <- cbind(kronecker(Rten2(Z1.2, one2), xt),
                      kronecker(Rten2(x1, one2), Zt.2),
                      kronecker(Rten2(Z1.2, one2), Zt.2))
-      colnames(Z_f1t_int) <- paste("Z_f1t_int",1:ncol(Z_f1t_int),sep=".")
+      colnames(Z_f1t_int) <- paste("Z_f1t_int", 1:ncol(Z_f1t_int), sep = ".")
       # g23u | g32u | g23b+g32b
       Z_f2t_int <- cbind(kronecker(Rten2(one1, Z2.2), xt),
                      kronecker(Rten2(one1, x2), Zt.2),
                      kronecker(Rten2(one1, Z2.2), Zt.2))
-      colnames(Z_f2t_int) <- paste("Z_f2t_int",1:ncol(Z_f2t_int),sep=".")
+      colnames(Z_f2t_int) <- paste("Z_f2t_int", 1:ncol(Z_f2t_int), sep=".")
       # g123u | g213u | g321u | g123b+g213b | g132b+g312b |
       # g231b+g321b | g1t+g2t+g3t
       Z_f12t_int <- cbind(kronecker(Rten2(Z1.3, x2), xt),
@@ -591,7 +651,8 @@ B_XZ_spt <- function(sp1,sp2,time,pordspt,psanova,decomspt,
                       kronecker(Rten2(Z1.3, x2), Zt.3),
                       kronecker(Rten2(x1, Z2.3), Zt.3),
                       kronecker(Rten2(Z1.3, Z2.3), Zt.3))
-      colnames(Z_f12t_int) <- paste("Z_f12t_int", 1:ncol(Z_f12t_int), sep = ".")
+      colnames(Z_f12t_int) <- paste("Z_f12t_int", 1:ncol(Z_f12t_int), 
+                                    sep = ".")
       Zspt <- NULL
       if(f1_main) Zspt <- cbind(Zspt,Z_f1_main)
       if(f2_main) Zspt <- cbind(Zspt,Z_f2_main)
@@ -600,15 +661,17 @@ B_XZ_spt <- function(sp1,sp2,time,pordspt,psanova,decomspt,
       if(f1t_int) Zspt <- cbind(Zspt,Z_f1t_int)
       if(f2t_int) Zspt <- cbind(Zspt,Z_f2t_int)
       if(f12t_int) Zspt <- cbind(Zspt,Z_f12t_int)
-      rm(X_f1_main,X_f2_main,X_ft_main,X_f12_int,X_f1t_int,X_f2t_int,X_f12t_int)
-      rm(Z_f1_main,Z_f2_main,Z_ft_main,Z_f12_int,Z_f1t_int,Z_f2t_int,Z_f12t_int)
-      rm(Xt,Zt,Zt.2,Z1.3,Z2.3,Zt.3)
+      rm(X_f1_main, X_f2_main, X_ft_main, X_f12_int, X_f1t_int, X_f2t_int, 
+         X_f12t_int)
+      rm(Z_f1_main, Z_f2_main, Z_ft_main, Z_f12_int, Z_f1t_int, Z_f2t_int,
+         Z_f12t_int)
+      rm(Xt, Zt, Zt.2, Z1.3, Z2.3, Zt.3)
     } else { # SPATIAL TREND. PS-ANOVA
       Xones <- Rten2(one1, one2)
       X_f1_main <- Rten2(x1, one2)
       X_f2_main <- Rten2(one1, x2)
       X_f12_int <- Rten2(x1, x2)
-      colnames(Xones) <- c("Intercept")
+      colnames(Xones) <- c("(Intercept)")
       colnames(X_f1_main) <- paste("X_f1_main", 1:ncol(X_f1_main), sep = ".")
       colnames(X_f2_main) <- paste("X_f2_main", 1:ncol(X_f2_main), sep = ".")
       colnames(X_f12_int) <- paste("X_f12_int", 1:ncol(X_f12_int), sep = ".")
